#ifndef COUNTER_HPP_INCLUDED
#define COUNTER_HPP_INCLUDED

#include <iostream>
#include <cstdint>

/** @addtogroup Types
* @{ */
/**
* @brief A Counter can be used, well as a Counter. You construct it with MaxIterations, the counter is marked as finished whenever the current Iterations reach, or exceed MaxIterations.
* You can use @ref Increment to increase, and @ref Reset to reset the current Iterations.
* You can also use @ref IsFinished to check the current state of the Counter.
*/
class Counter
{
    public:
        /**
        * @brief Default Counter constructor with MaxIterations as 0
		*/
        Counter();
        /**
        * @brief Constructs a Counter with
		* @param std::int32_t MaxIterations     Maximum amount of Iterations before the counter is Finished
		*/
        Counter(std::int32_t MaxIterations);
        /**
        * @brief Resets the Counter's current Iterations to 0
        * @par Description
        *   Resets the Counter's current Iterations to 0
        */
        void Reset();
        /**
        * @brief Increases the Counter's current Iterations by Amount, and returns true if the Counter isn't Finished
        * @return True if the Counter isn't Finished, after it increments the Counter's current Iterations
        */
        bool Increment(std::int32_t Amount = 1);
        /**
        * @brief Returns True if the Counter's current Iterations equals the MaxIterations Paramater passed in the Constructor
        * @return True if the Counter's current Iterations equals the MaxIterations Paramater passed in the Constructor
        */
        bool IsFinished() const;
        /**
        * @brief Returns the Counter's current Iterations
        * @return Returns the Counter's current Iterations
        */
        std::int32_t GetIterations() const;
        /**
        * @brief Returns the Counter's maximum Iterations
        * @return Returns the Counter's maximum Iterations
        */
        std::int32_t GetMaxIterations() const;
        friend std::ostream& operator<<(std::ostream& OS, const Counter& C);
        ~Counter();
    private:
        std::int32_t MaxIterations;
        std::int32_t Iterations;
};

/** @} */

#endif // COUNTER_HPP_INCLUDED
